/****************************************************************************/
/** browser.cpp                                                            **/
/** ---------------------------------------------------------------------- **/
/** NT UserBrowser sample (NETUI.. family)                                 **/
/** ---------------------------------------------------------------------- **/
/** June 14, 1998 by Oleg Kagan (oleg_kagan@geocities.com)                 **/
/**                                                                        **/
/** April 22, 1999 by Oleg Kagan (oleg_kagan@geocities.com)                **/
/**   DWORD OPENUSERBROWSER::Unknown2 have been changed to BOOL fCancelled **/
/**   due to information received from Sergei Fefelov (asf@chat.ru)        **/
/**                                                   Thank You, Sergei... **/
/**                                                                        **/
/** October 09, 1999 by Oleg Kagan (oleg_kagan@geocities.com)              **/
/**   Error handling added due to request from Alex (adou@lycosmail.com)   **/
/**   (see p_dwLastError parameter at BrowseUsersStr)                      **/
/**                                                     Good luck, Alex... **/
/****************************************************************************/

//============================================================================
#include "stdafx.h"
#pragma hdrstop
#include <malloc.h>

//============================================================================
#include "browser.h"

// TUserBrowser implementation
//============================================================================
TUserBrowser::TUserBrowser() :
	hBrowser(NULL), hNetUiDll(NULL)
{
	hNetUiDll = LoadLibrary("netui2.dll");
	if (!hNetUiDll) {
		MessageBox(
			NULL, "Can't load library", "Error!", MB_OK | MB_ICONSTOP
		) /*MessageBox*/;
		throw;
	} /*if*/

	pOpenUserBrowser = TOpenUserBrowserPtr(
		GetProcAddress(hNetUiDll, "OpenUserBrowser")
	) /*pOpenUserBrowser*/;
	if (!pOpenUserBrowser) {
		MessageBox(
			NULL, "Can't find OpenUserBrowser", "Error!", MB_OK | MB_ICONSTOP
		) /*MessageBox*/;
		throw;
	} /*if*/

	pEnumUserBrowserSelection = TEnumUserBrowserSelectionPtr(
		GetProcAddress(hNetUiDll, "EnumUserBrowserSelection")
	) /*pEnumUserBrowserSelection*/;
	if (!pEnumUserBrowserSelection) {
		MessageBox(
			NULL, "Can't find EnumUserBrowserSelection", "Error!", 
			MB_OK | MB_ICONSTOP
		) /*MessageBox*/;
		throw;
	} /*if*/

	pCloseUserBrowser = TCloseUserBrowserPtr(
		GetProcAddress(hNetUiDll, "CloseUserBrowser")
	) /*TCloseUserBrowserPtr*/;
	if (!pCloseUserBrowser) {
		MessageBox(
			NULL, "Can't find CloseUserBrowser", "Error!", MB_OK | MB_ICONSTOP
		) /*MessageBox*/;
		throw;
	} /*if*/
} /*TUserBrowser()*/

//----------------------------------------------------------------------------
std::wstring TUserBrowser::BrowseUsersStr(

	HWND hParent, 
	const std::wstring c_sTitle, 
	const std::wstring& c_sDomain, 
	DWORD uFlags,
	bool* p_bCancelled,
	size_t nBufferSize /*= 10 * 1024*/,
	DWORD* p_dwLastError /*= NULL*/

){
	if (p_dwLastError) *p_dwLastError = ERROR_SUCCESS;

	OPENUSERBROWSER OpenUserBrowser; {
		OpenUserBrowser.cbSize        = sizeof(OpenUserBrowser);
		OpenUserBrowser.fCancelled    = 0;
		OpenUserBrowser.Unknown2      = 1;
		OpenUserBrowser.hWndParent    = hParent;
		OpenUserBrowser.szTitle       = c_sTitle.c_str();
		OpenUserBrowser.szDomainName  = (
			c_sDomain.length() ? c_sDomain.c_str() : NULL
		) /*OpenUserBrowser.szDomainName*/;
		OpenUserBrowser.dwFlags       = uFlags;
		OpenUserBrowser.dwHelpId      = 0;
		OpenUserBrowser.szHelpFile    = NULL;
	} /*OpenUserBrowser*/

	std::wstring sResult;

	if (hBrowser = (*pOpenUserBrowser)(&OpenUserBrowser)) {
		if (p_bCancelled) *p_bCancelled = OpenUserBrowser.fCancelled;

		if (!OpenUserBrowser.fCancelled) {
			char* a_cBuffer = (char*)alloca(nBufferSize + 1);
			ENUMUSERBROWSER* pEnumUserBrowser = (ENUMUSERBROWSER*)a_cBuffer;
			DWORD uSize = nBufferSize;
			while (
				(*pEnumUserBrowserSelection)(
					hBrowser, pEnumUserBrowser, &uSize
				) /*(*pEnumUserBrowserSelection)*/
			){
				if (sResult.length()) sResult += L";";
				(sResult += pEnumUserBrowser->szDomainName) += L"\\";
				sResult += pEnumUserBrowser->szUserName;
			} /*while*/
			if (p_dwLastError) {
				*p_dwLastError = GetLastError();
				if (*p_dwLastError == ERROR_NO_MORE_ITEMS) 
					*p_dwLastError = ERROR_SUCCESS;
			} /*if*/
		} /*if*/
		(*pCloseUserBrowser)(hBrowser);
	}
	else {
		if (p_dwLastError) *p_dwLastError = GetLastError();
	} /*iff*/

	return sResult;
} /*BrowseUsersStr(HWND, const std::wstring, const std::wstring&,...)*/

